;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Loading this file installs CG/JS.
;; Please see readme.txt for more information
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :user)

(eval-when (compile eval load)
  (require :shell)
  (use-package :excl.shell))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; validate files

(let ((platform-file
       (directory
	(merge-pathnames
	 "PLATFORM-*"
	 (path-pathname *load-pathname*)))))
  (or platform-file
      (error "Could not find the PLATFORM file."))
  
  (when (> (length platform-file) 1)
    (error "There is more than one PLATFORM file: ~a." platform-file))
  (setq platform-file (car platform-file))
  
  (or (=~ "^PLATFORM-(.*)" (file-namestring platform-file))
      (error "Cannot parse the PLATFORM file."))
  
  (let ((expected-platform $1)
	(current-platform
	 (format nil "~a~a" (symbol-name sys::.os.) #+smp "smp" #-smp "")))
    (or (string-equal expected-platform current-platform)
	(error
	 "The current platform (~a) does not match the downloaded version (~a)."
	 current-platform expected-platform)))
  ;; We are on the correct platform, fall through...
  )

;; HTML files needed for the operation of CG/JS
;; (this variable's value is generated by build_cgjs.sh)
(defvar *html-files* '("cgjs.html" "ssl-not-found.html" "browser-not-supported.html" ))

(defvar *extra-files*)
(setq *extra-files* nil)

(dolist (file *html-files*)
  (setq file (merge-pathnames file *load-pathname*))
  (or (probe-file file)
      (error "~a does not exist." file))
  (push file *extra-files*))
;; All *html-files* exist...

;; These two are directories but the pathname is specified as
;; a file, so merging works below.
(defvar *cg2js-directory* (merge-pathnames "code.cg2js" *load-pathname*))
(defvar *idejs-directory* (merge-pathnames "code.idejs" *load-pathname*))

(dolist (d (list *cg2js-directory* *idejs-directory*))
  (or (probe-file d)
      (error "~a does not exist." d)))

;; MD5 checksums of the incoming files
;; (this variable's value is generated by build_cgjs.sh)
(defvar *checksums* 
'(
(#xfcd277b1768f25e8f0db05bf291b3f81 . "./code.cg2js/cg-scroll-bar-mixin.fasl")
(#xd302de6dc77f32825e68ec06db61826f . "./code.cg2js/cg-gesture-emulation.fasl")
(#xc3dee1263bdde9e07807a4bbb9fad0b5 . "./code.cg2js/cg-grid-widget.fasl")
(#xefac8ad96230b0394302098e11caf1bf . "./code.cg2js/cg-utility-dialog.fasl")
(#x8baac84889e97a88bbb0a1050d74bf56 . "./code.cg2js/cg-wrap-string.fasl")
(#xa442fae9b8a5361a88a885c203453117 . "./code.cg2js/cg-editable-text.fasl")
(#x1b9ee1505e9480787959fbbf7c2fb4d6 . "./code.cg2js/cg-base.fasl")
(#x5357b3cb004f8329dfd2361f139d7518 . "./code.cg2js/cg-lisp-text.fasl")
(#xca5f624ab15a559c50f70c061dcf4278 . "./code.cg2js/cg-intersect.fasl")
(#x8970d80788fb10365564efd5189bf034 . "./code.cg2js/cg-choose-printer.fasl")
(#x26b0a6c0e3a7c6156cf813fd50d8f99e . "./code.cg2js/cg-multi-line-lisp-text.fasl")
(#x5997c667534f8a77791087be6acb179e . "./code.cg2js/cg-outline-drag-and-drop.fasl")
(#x66aa7ac86e4a1e328c5cfd3dcc014083 . "./code.cg2js/cg-font-dialog.fasl")
(#xc6667b359fd9046df65a1081df3f6f1f . "./code.cg2js/cg-comtab.fasl")
(#xc33df1aa6e398aac461b1d29388b8640 . "./code.cg2js/cg-tooltip.fasl")
(#x43f0c44359b8545c9af5dce96b690467 . "./code.cg2js/cg-chart-or-plot.fasl")
(#x3fda50ff9139d799ca841df275c02902 . "./code.cg2js/cg-get-pixmap.fasl")
(#x35174cea60b7247583334058d2cea900 . "./code.cg2js/cg-status-bar.fasl")
(#xe0c1597ae328a126e4f072965460abdd . "./code.cg2js/cg-tab-control.fasl")
(#x6da3d9b80613a491a7b09c306cfb48d3 . "./code.cg2js/cg-icon-pixmap.fasl")
(#xdb265bee9609d9cc2538d6d55b2cd96b . "./code.cg2js/cg-list-view.fasl")
(#xe2d78f7ea5d7bf424dad20222e3765fb . "./code.cg2js/cg-text-edit-pane-file-io.fasl")
(#x922742297e2be9af9400d06ddd75d138 . "./code.cg2js/cg-scrolling-static-text.fasl")
(#xc3ec79109476549b2717449f939a3b5d . "./code.cg2js/cg-project-window.fasl")
(#xd039d3c5be09387f7877331a9afbe689 . "./code.cg2js/cg-multi-picture-button-drag-and-drop.fasl")
(#xc60b6627c5dcbe85d8ccf81b1761047e . "./code.cg2js/cg-drag-and-drop.fasl")
(#x27d32f84a400e2fda23fe666dc5cc727 . "./code.cg2js/cg-scaling-stream.fasl")
(#x7f03ca357a5847ebe0bfe6989ddeb552 . "./code.cg2js/cg-bitmap-stream.fasl")
(#x3193f87197bda4143bb0ce480f535816 . "./code.cg2js/cg-menu.fasl")
(#x5efe874b88655510ef54f56f44216fff . "./code.cg2js/cg-lisp-edit-pane.fasl")
(#x5c7d0057e96fe03488b325900bc4674d . "./code.cg2js/cg-intersect-posbox.fasl")
(#x5df5bec10e91768a34bfbab0b4405e8c . "./code.cg2js/cg-split-bar.fasl")
(#x04eaee81cc512a104003eb19d621a682 . "./code.cg2js/cg-drag-and-drop-image.fasl")
(#x94c4f29c4c2fe9f1c129c70d8e70a3ee . "./code.cg2js/cg-timer.fasl")
(#x9c479e079e3751edd04efc69c7298be7 . "./code.cg2js/cg-text-or-combo.fasl")
(#x626945f7f1ac0edf749ac7862e1af16a . "./code.cg2js/cg-picture-widget-palette.fasl")
(#xd1ab34d07d9a19aa05175d6fcf78600a . "./code.cg2js/cg-string-dialog.fasl")
(#x8bf955bf32c2c3cac1c3c7a7ab054ddc . "./code.cg2js/cg-clipboard-pixmap.fasl")
(#x3e5ee6399e9c89d0d3420fa7f4e73e1a . "./code.cg2js/cg-menu-tooltip.fasl")
(#x43fbdabfb499fb22d2a62714e7857ece . "./code.cg2js/cg-outline.fasl")
(#x791dbf3c21075e26e66b7d88b5af8f08 . "./code.cg2js/cg-lettered-menu.fasl")
(#xa6a185936a515f5c069892f7267c12bb . "./code.cg2js/cg-graphics-context.fasl")
(#xbe3cac9e43368ab6e8d34edfc15dc767 . "./code.cg2js/cg-item-list.fasl")
(#x2216084b5129f625ec7c12816e05185a . "./code.cg2js/cg-caret.fasl")
(#xe2be3cbb6a459de39004c52510657407 . "./code.cg2js/cg-printing.fasl")
(#xdb33f19efc1d8217638f92e6d2313fa8 . "./code.cg2js/cg-pixmap-printing.fasl")
(#x474c3691b132bd6b53887cbcb3732cf6 . "./code.cg2js/cg-progress-indicator.fasl")
(#x3a6ffae06fa1dfee522cb94ff390c33a . "./code.cg2js/cg-dde-utils.fasl")
(#xa8ebfa51abff9bc7a30486f39e34423b . "./code.cg2js/cg-plot-widget.fasl")
(#x3404e5c424a8dc72a1ac2043bc1ecd47 . "./code.cg2js/cg-os-window.fasl")
(#x60e8c4aa33daf1a0489c5c50f414a658 . "./code.cg2js/cg-calendar.fasl")
(#xb18e9285432ba01ad6f7fd78411fb187 . "./code.cg2js/cg-chart-widget.fasl")
(#x3125321f9dd6bc3acd7078393574917f . "./code.cg2js/cg-file-dialog.fasl")
(#x0a313668c6588d9d7f03fd48903d43f4 . "./code.cg2js/cg-nodes.fasl")
(#x400c3662079352e18d884a3ac26a424e . "./code.cg2js/cg-lamp.fasl")
(#xbdde620ebccd80fb809b1a9ae79d5157 . "./code.cg2js/cg-html-dialog.fasl")
(#x7b713eb525bf2282a5eec551fb217940 . "./code.cg2js/cg-multi-picture-button-tooltip.fasl")
(#xa7fcb9eb3c573224ee548d36b5e9d3c8 . "./code.cg2js/cg-class-slot-grid.fasl")
(#x602291e3706b728252feef2cebf66d8a . "./code.cg2js/cg-choice-list.fasl")
(#x7b8384d5660ec6574e19df4e6af10432 . "./code.cg2js/cg-multi-picture-button.fasl")
(#xb88d39a25d7588a90d82653b5433b6d3 . "./code.cg2js/cg-multi-line-editable-text.fasl")
(#xfeec5aef3e6dcf6084877c9a811a2e68 . "./code.cg2js/cg-hotspot.fasl")
(#xe15f4be085f4d39a28eed3dd109e7e1a . "./code.cg2js/cg-property.fasl")
(#xa1ed1b744c0752ba40caeba11f66d87a . "./code.cg2js/cg-dropping-outline.fasl")
(#x277faa65f13c4a6a59f73c2c14f39a9a . "./code.cg2js/cg-template-string.fasl")
(#x610aa6718d70dae8268ab8ebc756591b . "./code.cg2js/cg-palette.fasl")
(#x7e1ca5a48fec0832b941ee74d272d22c . "./code.cg2js/cg-selected-object.fasl")
(#x9213dc7bfe136cc28e9116336db526e7 . "./code.cg2js/cg-up-down-control.fasl")
(#x759ce0e662013e04b7be3be9159f5fe9 . "./code.cg2js/cg-keyboard-shortcuts.fasl")
(#xa6e6b185b13104eac73c44f4cdb3a7dc . "./code.cg2js/cg-layout.fasl")
(#x7168a38f24b4e959a8dba45c7333bba4 . "./code.cg2js/cg-get-position.fasl")
(#x7d6736b8b9de2c960a3c719e2b10abf2 . "./code.cg2js/cg-lisp-widget.fasl")
(#xb412737e40da8f85b6b533d66e09d063 . "./code.cg2js/cg-grid-widget-drag-and-drop.fasl")
(#x2a27a9e1cdb6c0441bc72f4091ef9146 . "./code.cg2js/cg-object-editor.fasl")
(#xdc8e706e80b927325b9da98a94d7fd56 . "./code.cg2js/cg-class-grid.fasl")
(#xfc7f411f65d54ab42573952564125f09 . "./code.cg2js/cg-yes-no-string.fasl")
(#x36daf4ee3c2d7f73f66804d17379c6a4 . "./code.cg2js/cg-toggling-widget.fasl")
(#xdb290298160efa6c1bcb9c1b0b9194e8 . "./code.cg2js/cg-text-edit-pane-mark.fasl")
(#x88b737685539db403d0f5d69a5e8ad01 . "./code.cg2js/cg-button.fasl")
(#x1d608d09c4b307426b6b77ba1b932f05 . "./code.cg2js/cg-object-editor-layout.fasl")
(#x51b5a284b767e833beb3b94710ad7fb9 . "./code.cg2js/cg-scroll-bar.fasl")
(#xe78e6512999ca2595240d0e3e23b0965 . "./code.cg2js/cg-combo-box.fasl")
(#x6513abe1af1331e656f9d62c112277b9 . "./code.cg2js/cg-web-browser.fasl")
(#xa32423852939151111969c855ddd8931 . "./code.cg2js/cg-drawable.fasl")
(#x89b577bae1d32574095975f6ae74ce69 . "./code.cg2js/cg-cursor-pixmap.fasl")
(#x34abe2b3970dcfac2f403f52d3b0ed18 . "./code.cg2js/cg-message-dialog.fasl")
(#x71ae2ef6bdbf50c3b387696216eead94 . "./code.cg2js/cg-text-widget.fasl")
(#x8fff8cd3af63e0c701897af2a613fe75 . "./code.cg2js/cg-static-text.fasl")
(#x7dbf3707bdb935dbcc99a77419a7ff30 . "./code.cg2js/cg-class-support.fasl")
(#x5c2dc02c681b17ca5ba10faa9576f7fa . "./code.cg2js/cg.fasl")
(#x9684ea77f4c105e2d9412598f1195af6 . "./code.cg2js/cg-directory-dialog.fasl")
(#x66052254bddd1fa884f07d6be6bd02ff . "./code.cg2js/cg-clipboard.fasl")
(#x77f7621d082192a934c14483b20a1d30 . "./code.cg2js/cg-drawable-clipboard.fasl")
(#x1fa18a3d8c79ae738a278168a86469c6 . "./code.cg2js/cg-bitmap-pane.fasl")
(#x72cc92223dd7a7ff906fe2e36ac38ee2 . "./code.cg2js/cg-bitmap-pane-clipboard.fasl")
(#x4057f188082a5f004c51a74c1553c569 . "./code.cg2js/cg-check-box.fasl")
(#x9f09c6a1d8bbd74627729563d23cefde . "./code.cg2js/cg-color-dialog.fasl")
(#x2043cc7d3537c808e80253c420d37f83 . "./code.cg2js/cg-text-edit-pane.fasl")
(#xaca04d6d6e99597511d3cebd14e58917 . "./code.cg2js/cg-dialog-item.fasl")
(#xf833ca9729b9a041a397b18a9fb9a816 . "./code.cg2js/cg-edit-in-place.fasl")
(#xfdd5fc3a044883ebc3ac339f1794e025 . "./code.cg2js/cg-tray.fasl")
(#x9a303237ff343a911cde7139dfe68fa7 . "./code.cg2js/cg-paren-matching.fasl")
(#x5daac6a93d73d2c88a2efd5e94c4e803 . "./code.cg2js/cg-player.fasl")
(#x8321f3da7c626a9a20a571f809f17a6f . "./code.cg2js/cg-yes-no-list.fasl")
(#x706ccb36c6836467118966e7c63b040e . "./code.cg2js/cg-common-control.fasl")
(#xd95bacecdedbb1f380f96bb7920e9fcf . "./code.cg2js/cg-pixmap-file-io.fasl")
(#x86f047bd68653688ed560686844f56f2 . "./code.cg2js/cg-os-widget.fasl")
(#xebf39cd5158831c74c17110f137eabf6 . "./code.cg2js/cg-sample-file-menu.fasl")
(#x1bb6eed248966d8f61840773ff7110e2 . "./code.cg2js/cg-trackbar.fasl")
(#xd912b996604af924f3ac0934d783b8d2 . "./code.cg2js/cg-find-string-dialog.fasl")
(#xbdf6380afaedfb819d844f13c729fce6 . "./code.cg2js/cg-html-widget.fasl")
(#xf4337330ef2e70d0bf40b549d59cc108 . "./code.cg2js/cg-directory-dialog-os.fasl")
(#xe3f158fc510f21a44da643a3e7a0f913 . "./code.cg2js/cg-acache.fasl")
(#xe2406e910cd825cc0d179da3c85665e7 . "./code.cg2js/cg-shortcut-menu.fasl")
(#x0ef8fbd9759e794f8bd6048a3a97f358 . "./code.cg2js/cg-pixmap-widget.fasl")
(#x3ceab749a16e7c5fddf055a19ef29579 . "./code.cg2js/cg-icon.fasl")
(#x5841a27776c12c70ebc808984c7a674a . "./code.cg2js/cg-pixmap.fasl")
(#xf7862d227f8d8ae0f7442b17eaafa519 . "./code.cg2js/cg-clipboard-stack.fasl")
(#x5dd3d01a99afeedf8da71893ca1a5268 . "./code.cg2js/cg-outline-edit-in-place.fasl")
(#x1325b0f122512741ae05187593816363 . "./code.cg2js/cg-toolbar.fasl")
(#x9472edc9dc9207d6ffbf68bcf92ecf60 . "./code.cg2js/cg-radio-button.fasl")
(#xee9316d66fb511884ec74a5a4829dce0 . "./code.cg2js/cg-curve.fasl")
(#x60ac2365f118ec0dc3090ad93d7f218f . "./code.cg2js/cg-picture-widget.fasl")
(#x492cbae6e529979ea25a3a1371764d0a . "./code.cg2js/cg-dev.fasl")
(#xd148ac4c4d0f30d4fa0010a40d7cfee2 . "./code.cg2js/cg-group-box.fasl")
(#xd358dabf72742f10d5adfe88e5211252 . "./code.cg2js/cg-fill-texture.fasl")
(#x384016f60911e902d8801c382446e40c . "./code.cg2js/cg-pixmap-rotate.fasl")
(#x73e75fd62fc890961a09d9235749543a . "./code.idejs/ide-keyshort.fasl")
(#x69ad32fddee00171192dd9d88c75f623 . "./code.idejs/ide-form.fasl")
(#xeb11f02e88f9becb110941547871720e . "./code.idejs/ide-find-in-files.fasl")
(#x7b61515b4b45969093b5018d2060bd58 . "./code.idejs/ide-class-grapher.fasl")
(#xfa414b12ad407241a06e6fbe83e2ac12 . "./code.idejs/ide-base.fasl")
(#xd926aa467d1171009106c7d82b8c68f0 . "./code.idejs/ide-class-browser.fasl")
(#x5a623f96fd579d07f7061f730d0d5e32 . "./code.idejs/ide-find-definitions.fasl")
(#x3817d8cc3c6ebfe8244c8e047763419b . "./code.idejs/ide-package-list.fasl")
(#x6be15e30f5711af329e2590f065a960a . "./code.idejs/ide-grapher.fasl")
(#xb56243ea75ae73dbb16b789d7c54a8d7 . "./code.idejs/ide-dev.fasl")
(#x914353e2fa8b43f3ee6858538d1b4db6 . "./code.idejs/ide.fasl")
(#xa6641d3affcae06cab621e440ce0c8db . "./code.idejs/ide-trace.fasl")
(#xd5134454e0cd84220de8bc80e4931dad . "./code.idejs/ide-apropos.fasl")
(#xb8cd70264799796fd0cf03edb675df76 . "./code.idejs/ide-inspector.fasl")
(#x2d6cd15e7a35a35878e3c77bacbb49b4 . "./code.idejs/ide-acache.fasl")
(#x9959352fa890aba02b64e14dcba853f4 . "./code.idejs/ide-project.fasl")
(#xc4d2207388393f5b9377e514420affd9 . "./code.idejs/ide-window-list.fasl")
(#xf65dd9181f46ec23666c76cb8338411e . "./code.idejs/ide-intro.fasl")
(#x652edc3de5286490619fa50fb07668b7 . "./code.idejs/ide-clipboard.fasl")
(#x4bd7046c632f1bc4bd0e30396d2cc69c . "./code.idejs/ide-editor.fasl")
(#x284906efd645f4cc601bb163e204bf57 . "./code.idejs/ide-process-browser.fasl")
(#x3c2905f2b856b80edd29fd50eac5bd34 . "./code.idejs/ide-patch-dialog.fasl")
(#x96e7da73e9e5887f9aca8e6e284475e7 . "./code.idejs/ide-navigator.fasl")
(#x9a8f50ffdba20a10156ef75920aa924b . "./cgjs.html")
(#xdac1739373835d1a63cbafa2c3e0b06a . "./ssl-not-found.html")
(#x90c6d2a65504d45266ad9fa3a49be2f9 . "./browser-not-supported.html"))
 )

(let ((files-ok
       (dolist (checksum *checksums* t)
	 (when (/= (car checksum)
		   (md5-file
		    (merge-pathnames (cdr checksum) *load-pathname*)))
	   (format t "Incorrect checksum on file: ~a" (cdr checksum))
	   (return nil)))))
  (cond
   ((not files-ok)
    (error "The extracted files failed the checksum test.  ~
Please download and/or extract again."))
   (t
    (format t "Checksums on files are OK...~%"))))

;; We have validated all the files are there...

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; check for needed patches

(eval-when (compile eval load) (require :build))

(defvar *required-patches* '(#+macosx ("build" . 3)))

(dolist (required-patch *required-patches*)
  (let ((existing-patch (assoc (car required-patch)
			    (cdr (assoc :system sys::*patches*))
			    :test #'string=)))
    (when (null existing-patch)
      (error "Could not find patch info for the ~s module."
	     (car required-patch)))
    (when (not (>= (second existing-patch) (cdr required-patch)))
      ;; The build patch number is NOT >= to the one we require
      (error "~
Expected version ~d of ~s but have version ~d.  Please update ~
your lisp with update.~a and then do the CG/JS installation again."
	     (cdr required-patch) (car required-patch)
	     (second existing-patch)
	     #+mswindows "exe"
	     #-mswindows "sh"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; copy files

(defun copy (source dest)
  (cond
   ((file-directory-p source)
    (format t "Copy directory ~a~%" (enough-namestring source))
    (copy-directory source dest :quiet t))
   (t
    (format t "Copy file ~a~%" (enough-namestring source))
    (sys:copy-file source dest :verbose nil))))

(defvar *sys* (translate-logical-pathname (pathname "sys:")))

(cond
 ((excl::pathname-equalp (truename *sys*)
			 (path-pathname *load-truename*))
  (format t "Files are already in place, no copying requiring.~%"))
 (t ;; copy files, replacing them with new ones if they already exist
  (let (dest)
    (dolist (from *extra-files*)
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      (when (probe-file dest) (delete-file dest))
      (copy from dest))
    
    (dolist (from (list *cg2js-directory* *idejs-directory*))
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      ;; now make it a directory pathname
      (setq dest (pathname-as-directory dest))

      (when (probe-file dest) (delete-directory-and-files dest))
      (copy (pathname-as-directory from) dest)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; update develenv.cl to load CG/JS before the original version of CG

(defvar *setup-cgjs-loading* "sys:setup-cgjs-loading.cl")

;; Because users might use an ANSI lisp to install, have to make sure
;; the resulting file can be loaded into an ANSI and Modern lisp.
(with-open-file (s *setup-cgjs-loading* :direction :output
		 :if-exists :supersede)
  (format s "~
;; This file was created by install-cgjs.cl.
;; Please do not edit it.

")
  (dolist (dir (list "code.cg2js" "code.idejs"))
    (format s "
(let ((path (make-pathname
             :host \"sys\"
             :directory '(:relative \"~a\")
	     :type \"fasl\"
	     )))
  (format t \";; *require-search-list*: adding: ~~s~%\" path)
  (push path sys:*require-search-list*))

"
	    dir)))

(defvar *develenv.cl* (pathname "sys:develenv.cl"))

(when (not (probe-file *develenv.cl*))
  (setf (file-contents *develenv.cl*)
    ";; This file is loaded before images are rebuilt with update.sh/update.exe

"))

(let ((contents (file-contents *develenv.cl*)))
  (cond
   ((match-re
     (format nil "^#\\|\\|# \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl already correctly loads CG/JS.~%"))
   ((match-re
     (format nil "^#\\+ignore \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl: CG/JS disabled; will enable.~%")
    (setf (file-contents *develenv.cl*)
      (replace-re  contents
		  (format nil "^#\\+ignore *\\(load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  (format nil "#||# (load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  :multiple-lines t
		  :count 1)))
   (t
    (format t "develenv.cl: adding CG/JS initialization.~%")
    (setf (file-contents *develenv.cl*)
      (concatenate 'simple-string
	contents
	(format nil "~

;; To disable the use of CG/JS and go back to the originally released
;; (and possibly patched) version of CG/IDE, change the #||# to
;; #+ignore on the following line and the rebuild images by running
;; update.sh or update.exe, depending on your platform:
#||# (load \"sys:~a\")~%~%"
		(file-namestring *setup-cgjs-loading*)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; success!

(let ((msg (format nil "~
CG/JS installation successful!

NOTE: to use CG/JS you must rebuild images by running 'update.~a'
      in the Allegro CL directory (~a)."
		   #+mswindows "exe"
		   #-mswindows "sh"
		   *sys*)))
  #-mswindows (format t "~&~%~a~%" msg)
  #+mswindows
  (let ((noninteractive (sys:getenv "ACL_NONINTERACTIVE")))
    (when (or (null noninteractive)
	      (string= "" noninteractive))
      (excl::internal-message-box msg "CG/JS Installation" :type :ok))))
